@@ -29,6 +29,7 @@
 #include <akonadi/collectionfetchjob.h>
 #include <akonadi/collectionfetchscope.h>
 #include <akonadi/entityhiddenattribute.h>
+#include <akonadi/indexpolicyattribute.h>
 #include <akonadi/item.h>
 #include <akonadi/itemfetchjob.h>
 #include <akonadi/itemfetchscope.h>
@@ -48,6 +49,7 @@
 #include <KMessageBox>
 #include <KStandardDirs>
 #include <KIdleTime>
+#include <KNotification>
 
 #include <Soprano/Vocabulary/NAO>
 
@@ -68,9 +70,16 @@
 
 using namespace Akonadi;
 
-static inline bool entityIsHidden( const Entity &entity )
+static inline bool indexingDisabled( const Collection &collection )
 {
-  return entity.hasAttribute<EntityHiddenAttribute>();
+  if ( collection.hasAttribute<EntityHiddenAttribute>() )
+    return true;
+
+  IndexPolicyAttribute *indexPolicy = collection.attribute<IndexPolicyAttribute>();
+  if ( indexPolicy && !indexPolicy->indexingEnabled() )
+      return true;
+
+  return false;
 }
 
 NepomukFeederAgentBase::NepomukFeederAgentBase(const QString& id) :
@@ -108,6 +117,10 @@ NepomukFeederAgentBase::NepomukFeederAgentBase(const QString& id) :
 
   checkOnline();
   QTimer::singleShot( 0, this, SLOT(selfTest()) );
+
+  mProcessPipelineTimer.setInterval( 0 );
+  mProcessPipelineTimer.setSingleShot( true );
+  connect( &mProcessPipelineTimer, SIGNAL(timeout()), SLOT(processPipeline()) );
 }
 
 NepomukFeederAgentBase::~NepomukFeederAgentBase()
@@ -119,14 +132,12 @@ NepomukFeederAgentBase::~NepomukFeederAgentBase()
 
 void NepomukFeederAgentBase::itemAdded(const Akonadi::Item& item, const Akonadi::Collection& collection)
 {
-  if ( entityIsHidden( collection ) )
+  if ( indexingDisabled( collection ) )
     return;
 
   if ( item.hasPayload() ) {
-    const QUrl graph = createGraphForEntity( item );
-    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
-                             QUrl( QString::number( item.id() ) ), graph );
-    updateItem( item, graph );
+    mItemPipeline.enqueue( item );
+    mProcessPipelineTimer.start();
   } else {
     const ItemFetchScope scope = fetchScopeForCollection( collection );
     if ( scope.fullPayload() || !scope.payloadParts().isEmpty() ) {
@@ -140,16 +151,13 @@ void NepomukFeederAgentBase::itemAdded(const Akonadi::Item& item, const Akonadi:
 
 void NepomukFeederAgentBase::itemChanged(const Akonadi::Item& item, const QSet< QByteArray >& partIdentifiers)
 {
-  if ( entityIsHidden( item.parentCollection() ) )
+  if ( indexingDisabled( item.parentCollection() ) )
     return;
   // TODO: check part identfiers if anything interesting changed at all
   if ( item.hasPayload() ) {
     removeEntityFromNepomuk( item );
-
-    const QUrl graph = createGraphForEntity( item );
-    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
-                             QUrl( QString::number( item.id() ) ), graph );
-    updateItem( item, graph );
+    mItemPipeline.enqueue( item );
+    mProcessPipelineTimer.start();
   } else {
     const Collection collection = item.parentCollection();
     const ItemFetchScope scope = fetchScopeForCollection( collection );
@@ -170,6 +178,8 @@ void NepomukFeederAgentBase::itemRemoved(const Akonadi::Item& item)
 void NepomukFeederAgentBase::collectionAdded(const Akonadi::Collection& collection, const Akonadi::Collection& parent)
 {
   Q_UNUSED( parent );
+  if ( indexingDisabled( collection ) )
+    return;
   updateCollection( collection, createGraphForEntity( collection ) );
 }
 
@@ -177,6 +187,8 @@ void NepomukFeederAgentBase::collectionChanged(const Akonadi::Collection& collec
 {
   Q_UNUSED( partIdentifiers );
   removeEntityFromNepomuk( collection );
+  if ( indexingDisabled( collection ) )
+    return;
   updateCollection( collection, createGraphForEntity( collection ) );
 }
 
@@ -206,7 +218,7 @@ void NepomukFeederAgentBase::collectionsReceived(const Akonadi::Collection::List
     if ( !mMimeTypeChecker.isWantedCollection( collection ) )
       continue;
 
-    if ( entityIsHidden( collection ) )
+    if ( indexingDisabled( collection ) )
       continue;
 
     mCollectionQueue.append( collection );
@@ -221,6 +233,7 @@ void NepomukFeederAgentBase::processNextCollection()
 {
   if ( mCurrentCollection.isValid() )
     return;
+  mTotalAmount = 0;
   if ( mCollectionQueue.isEmpty() ) {
     emit fullyIndexed();
     return;
@@ -235,7 +248,6 @@ void NepomukFeederAgentBase::processNextCollection()
   connect( itemFetch, SIGNAL(itemsReceived(Akonadi::Item::List)), SLOT(itemHeadersReceived(Akonadi::Item::List)) );
   connect( itemFetch, SIGNAL(result(KJob*)), SLOT(itemFetchResult(KJob*)) );
   ++mPendingJobs;
-  mTotalAmount = 0;
 }
 
 void NepomukFeederAgentBase::itemHeadersReceived(const Akonadi::Item::List& items)
@@ -277,7 +289,7 @@ void NepomukFeederAgentBase::itemFetchResult(KJob* job)
     kDebug() << job->errorString();
 
   --mPendingJobs;
-  if ( mPendingJobs == 0 ) {
+  if ( mPendingJobs == 0 && mItemPipeline.isEmpty() ) {
     mCurrentCollection = Collection();
     emit status( Idle, i18n( "Indexing completed." ) );
     processNextCollection();
@@ -291,13 +303,9 @@ void NepomukFeederAgentBase::itemsReceived(const Akonadi::Item::List& items)
   foreach ( Item item, items ) {
     // we only get here if the item is not anywhere in Nepomuk yet, so no need to delete it
     item.setParentCollection( mCurrentCollection );
-    const QUrl graph = createGraphForEntity( item );
-    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
-                             QUrl( QString::number( item.id() ) ), graph );
-    updateItem( item, graph );
+    mItemPipeline.enqueue( item );
   }
-  mProcessedAmount += items.count();
-  emit percent( (mProcessedAmount * 100) / (mTotalAmount * 100) );
+  mProcessPipelineTimer.start();
 }
 
 void NepomukFeederAgentBase::notificationItemsReceived(const Akonadi::Item::List& items)
@@ -308,12 +316,9 @@ void NepomukFeederAgentBase::notificationItemsReceived(const Akonadi::Item::List
       continue;
     }
     removeEntityFromNepomuk( item );
-
-    const QUrl graph = createGraphForEntity( item );
-    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
-                             QUrl( QString::number( item.id() ) ), graph );
-    updateItem( item, graph );
+    mItemPipeline.enqueue( item );
   }
+  mProcessPipelineTimer.start();
 }
 
 void NepomukFeederAgentBase::tagsFromCategories(NepomukFast::Resource& resource, const QStringList& categories)
@@ -373,8 +378,11 @@ void NepomukFeederAgentBase::selfTest()
   // try to obtain a Strigi index manager with a Soprano backend
   if ( !mStrigiIndexManager && mNeedsStrigi ) {
     mStrigiIndexManager = Strigi::IndexPluginLoader::createIndexManager( "nepomukbackend", 0 );
-    if ( !mStrigiIndexManager )
+    if ( !mStrigiIndexManager ) {
       errorMessages.append( i18n( "Nepomuk backend for Strigi is not available." ) );
+      foreach ( const std::string &plugin, Strigi::IndexPluginLoader::indexNames() )
+        kWarning() << "Available plugins are: " << plugin.c_str();
+    }
   }
 
   if ( errorMessages.isEmpty() ) {
@@ -405,7 +413,7 @@ void NepomukFeederAgentBase::selfTest()
   emit status( Broken, i18n( "Nepomuk not operational" ) );
   if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.kde.pim.nepomukfeeder.selftestreport" ) ) )
     return;
-  KMessageBox::error( 0, message, i18n( "Nepomuk Indexing Disabled" ), KMessageBox::Notify | KMessageBox::AllowLink );
+  KNotification::event( KNotification::Warning, i18n( "Nepomuk Indexing Disabled" ), message );
   QDBusConnection::sessionBus().unregisterService( QLatin1String( "org.kde.pim.nepomukfeeder.selftestreport" ) );
 }
 
@@ -518,6 +526,14 @@ void NepomukFeederAgentBase::checkOnline()
     setOnline( mSelfTestPassed );
   else
     setOnline( mSelfTestPassed && mSystemIsIdle );
+
+  if ( isOnline() && !mItemPipeline.isEmpty() ) {
+    if ( mCurrentCollection.isValid() )
+      emit status( AgentBase::Running, i18n( "Indexing collection '%1'...", mCurrentCollection.name() ) );
+    else
+      emit status( AgentBase::Running, i18n( "Indexing recent changes..." ) );
+    mProcessPipelineTimer.start();
+  }
 }
 
 void NepomukFeederAgentBase::systemIdle()
@@ -541,4 +557,40 @@ void NepomukFeederAgentBase::systemResumed()
   checkOnline();
 }
 
+void NepomukFeederAgentBase::processPipeline()
+{
+  static bool processing = false; // guard against sub-eventloop reentrancy
+  if ( processing )
+    return;
+  processing = true;
+  if ( !mItemPipeline.isEmpty() && isOnline() ) {
+    const Akonadi::Item &item = mItemPipeline.dequeue();
+    const QUrl graph = createGraphForEntity( item );
+    mNrlModel->addStatement( item.url(), Akonadi::ItemSearchJob::akonadiItemIdUri(),
+                             QUrl( QString::number( item.id() ) ), graph );
+    updateItem( item, graph );
+    ++mProcessedAmount;
+    if ( (mProcessedAmount % 10) == 0 && mTotalAmount > 0 && mProcessedAmount <= mTotalAmount )
+      emit percent( (mProcessedAmount * 100) / mTotalAmount );
+    if ( !mItemPipeline.isEmpty() ) {
+      // go to eventloop before processing the next one, otherwise we miss the idle status change
+      mProcessPipelineTimer.start();
+    }
+  }
+  processing = false;
+
+  if ( mItemPipeline.isEmpty() )
+    emit status( AgentBase::Idle, QString() );
+
+  if ( !isOnline() )
+    return;
+
+  if ( mPendingJobs == 0 && mCurrentCollection.isValid() && mItemPipeline.isEmpty() ) {
+    mCurrentCollection = Collection();
+    emit status( Idle, i18n( "Indexing completed." ) );
+    processNextCollection();
+    return;
+  }
+}
+
 #include "nepomukfeederagentbase.moc"
