@@ -1,6 +1,7 @@
 /*  This file is part of the KDE project
     Copyright (C) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.net
     Author: Kevin Krammer, krake@kdab.com
+    Copyright (C) 2011 Kevin Krammer, kevin.krammer@gmx.at
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -38,7 +39,6 @@
 #include <akonadi/itemcreatejob.h>
 #include <akonadi/itemdeletejob.h>
 #include <akonadi/itemfetchscope.h>
-#include <akonadi/monitor.h>
 #include <akonadi/session.h>
 #include <akonadi/kmime/messagestatus.h>
 
@@ -60,28 +60,17 @@ class ImapCacheCollectionMigrator::Private
   ImapCacheCollectionMigrator *const q;
 
   public:
-    Private( ImapCacheCollectionMigrator *parent, MixedMaildirStore *store )
-      : q( parent ), mStore( store ), mHiddenSession( 0 ),
+    Private( ImapCacheCollectionMigrator *parent )
+      : q( parent ),
         mImportNewMessages( false ), mImportCachedMessages( false ),
         mRemoveDeletedMessages( false ), mDeleteImportedMessages( false ),
         mItemProgress( -1 )
     {
     }
 
-    ~Private()
-    {
-      delete mHiddenSession;
-    }
-
-    Collection cacheCollection( const Collection &collection ) const;
-
     bool isUnsubscribedImapFolder( const Collection &collection, QString &idPath ) const;
 
   public:
-    MixedMaildirStore *mStore;
-
-    Session *mHiddenSession;
-
     Collection mCurrentCollection;
     Item::List mItems;
     UidHash mUidHash;
@@ -114,23 +103,6 @@ class ImapCacheCollectionMigrator::Private
     void unsubscribeCollectionsResult( KJob *job );
 };
 
-Collection ImapCacheCollectionMigrator::Private::cacheCollection( const Collection &collection ) const
-{
-  if ( collection.parentCollection() == Collection::root() ) {
-    Collection cache;
-    cache.setRemoteId( q->topLevelFolder() );
-    cache.setParentCollection( mStore->topLevelCollection() );
-    return cache;
-  }
-
-  const QString remoteId = collection.remoteId().mid( 1 );
-
-  Collection cache;
-  cache.setRemoteId( remoteId );
-  cache.setParentCollection( cacheCollection( collection.parentCollection() ) );
-  return cache;
-}
-
 bool ImapCacheCollectionMigrator::Private::isUnsubscribedImapFolder( const Collection &collection, QString &idPath ) const
 {
   if ( collection.parentCollection() == Collection::root() ) {
@@ -196,6 +168,25 @@ void ImapCacheCollectionMigrator::Private::fetchItemsResult( KJob *job )
     kDebug( KDE_DEFAULT_DEBUG_AREA ) << mTagListHash.count() << "items have tags";
   }
 
+  // filter out items we don't process later on
+  const int oldCount = mItems.count();
+
+  Item::List::iterator itemIt = mItems.begin();
+  while ( itemIt != mItems.end() ) {
+    const QString storeRemoteId = (*itemIt).remoteId();
+    if ( mImportCachedMessages ||
+         ( mImportNewMessages && !mUidHash.contains( storeRemoteId ) ) ||
+         mTagListHash.contains( storeRemoteId ) ) {
+      ++itemIt;
+    } else {
+      itemIt = mItems.erase( itemIt );
+    }
+  }
+
+  if ( oldCount != mItems.count() ) {
+    kDebug( KDE_DEFAULT_DEBUG_AREA ) << "After filtering:" << mItems.count() << "items remaining";
+  }
+
   mItemProgress = -1;
   emit q->progress( 0, mItems.count(), 0 );
   processNextItem();
@@ -222,7 +213,7 @@ void ImapCacheCollectionMigrator::Private::processNextItem()
     return;
   }
 
-  const Item item = mItems.front();
+  Item item = mItems.front();
   mItems.pop_front();
 
   // don't import items that are marked deleted. These come from normal/online IMAP caches
@@ -235,9 +226,30 @@ void ImapCacheCollectionMigrator::Private::processNextItem()
     return;
   }
 
-  FileStore::ItemFetchJob *job = mStore->fetchItem( item );
-  job->fetchScope().fetchFullPayload( true );
-  connect( job, SIGNAL( result( KJob* ) ), q, SLOT( fetchItemResult( KJob* ) ) );
+  const QString storeRemoteId = item.remoteId();
+  if ( mImportCachedMessages || ( mImportNewMessages && !mUidHash.contains( storeRemoteId ) ) ) {
+    FileStore::ItemFetchJob *job = q->store()->fetchItem( item );
+    job->fetchScope().fetchFullPayload( true );
+    connect( job, SIGNAL( result( KJob* ) ), q, SLOT( fetchItemResult( KJob* ) ) );
+  } else if ( mTagListHash.contains( storeRemoteId ) ) {
+    ItemCreateJob *createJob = 0;
+
+    if ( !mUidHash.contains( storeRemoteId ) ) {
+      item.setRemoteId( QString() );
+      createJob = new ItemCreateJob( item, mCurrentCollection );
+    } else {
+      const QString uid = mUidHash[ storeRemoteId ];
+      item.setRemoteId( uid );
+      createJob = new ItemCreateJob( item, mCurrentCollection, q->hiddenSession() );
+    }
+
+    createJob->setProperty( "storeRemoteId", storeRemoteId );
+    createJob->setProperty( "storeParentCollection", QVariant::fromValue<Collection>( item.parentCollection() ) );
+    connect( createJob, SIGNAL( result( KJob* ) ), q, SLOT( itemCreateResult( KJob* ) ) );
+
+  } else {
+    QMetaObject::invokeMethod( q, "processNextItem", Qt::QueuedConnection );
+  }
 }
 
 void ImapCacheCollectionMigrator::Private::processNextDeletedUid()
@@ -268,7 +280,7 @@ void ImapCacheCollectionMigrator::Private::processNextDeletedUid()
   item.setMimeType( KMime::Message::mimeType() );
   item.setRemoteId( uid );
 
-  ItemCreateJob *createJob = new ItemCreateJob( item, mCurrentCollection, mHiddenSession );
+  ItemCreateJob *createJob = new ItemCreateJob( item, mCurrentCollection, q->hiddenSession() );
   connect( createJob, SIGNAL( result( KJob* ) ), q, SLOT( itemDeletePhase1Result( KJob* ) ) );
 }
 
@@ -310,7 +322,7 @@ void ImapCacheCollectionMigrator::Private::fetchItemResult( KJob *job )
 //                                     << "flags=" << item.flags();
   } else if ( mImportCachedMessages ) {
     item.setRemoteId( uid );
-    createJob = new ItemCreateJob( item, mCurrentCollection, mHiddenSession );
+    createJob = new ItemCreateJob( item, mCurrentCollection, q->hiddenSession() );
 
 //    kDebug( KDE_DEFAULT_DEBUG_AREA ) << "synchronized cacheItem: remoteId=" << item.remoteId()
 //                                     << "mimeType=" << item.mimeType()
@@ -365,7 +377,7 @@ void ImapCacheCollectionMigrator::Private::itemCreateResult( KJob *job )
       Item cacheItem = item;
       cacheItem.setRemoteId( storeRemoteId );
       cacheItem.setParentCollection( storeCollection );
-      FileStore::ItemDeleteJob *deleteJob = mStore->deleteItem( cacheItem );
+      FileStore::ItemDeleteJob *deleteJob = q->store()->deleteItem( cacheItem );
       connect( deleteJob, SIGNAL( result( KJob* ) ), q, SLOT( cacheItemDeleteResult( KJob* ) ) );
     } else {
       processNextItem();
@@ -444,11 +456,9 @@ void ImapCacheCollectionMigrator::Private::unsubscribeCollectionsResult( KJob *j
   }
 }
 
-ImapCacheCollectionMigrator::ImapCacheCollectionMigrator( const AgentInstance &resource, MixedMaildirStore *store, QObject *parent )
-  : AbstractCollectionMigrator( resource, parent ), d( new Private( this, store ) )
+ImapCacheCollectionMigrator::ImapCacheCollectionMigrator( const AgentInstance &resource, const QString &resourceName, MixedMaildirStore *store, QObject *parent )
+  : AbstractCollectionMigrator( resource, resourceName, store, parent ), d( new Private( this ) )
 {
-  d->mHiddenSession = new Session( resource.identifier().toAscii() );
-
   connect( this, SIGNAL( migrationFinished( Akonadi::AgentInstance, QString ) ),
            SLOT( unsubscribeCollections() ) );
 }
@@ -462,13 +472,13 @@ void ImapCacheCollectionMigrator::setMigrationOptions( const MigrationOptions &o
 {
   MigrationOptions actualOptions = options;
 
-  if ( d->mStore == 0 ) {
+  if ( store() == 0 ) {
     emit message( KMigratorBase::Skip,
                   i18nc( "@info:status", "No cache for account %1 available",
-                         resource().name() ) );
+                         resourceName() ) );
     kWarning() << "No store for folder" << topLevelFolder()
                << "so only config migration (instead of" << options << ") for"
-               << resource().identifier() << resource().name();
+               << resource().identifier() << resourceName();
     actualOptions = ConfigOnly;
   }
 
@@ -536,7 +546,7 @@ void ImapCacheCollectionMigrator::migrateCollection( const Collection &collectio
   // check that we don't get entered while we are still processing
   Q_ASSERT( !d->mCurrentCollection.isValid() );
 
-  Q_ASSERT( d->mStore != 0 );
+  Q_ASSERT( store() != 0 );
 
   if ( collection.parentCollection() == Collection::root() ) {
     emit status( QString() );
@@ -546,8 +556,9 @@ void ImapCacheCollectionMigrator::migrateCollection( const Collection &collectio
 
   kDebug( KDE_DEFAULT_DEBUG_AREA ) << "Akonadi collection remoteId=" << collection.remoteId() << ", parent=" << collection.parentCollection().remoteId();
 
-  Collection cache = d->cacheCollection( collection );
+  Collection cache = currentStoreCollection();
   kDebug( KDE_DEFAULT_DEBUG_AREA ) << "Cache collection remoteId=" << cache.remoteId() << ", parent=" << cache.parentCollection().remoteId();
+  kDebug( KDE_DEFAULT_DEBUG_AREA ) << "folderId=" << folderId << "imapIdPath=" << imapIdPath;
 
   d->mDeletedUids.clear();
   if ( d->mRemoveDeletedMessages ) {
@@ -564,12 +575,12 @@ void ImapCacheCollectionMigrator::migrateCollection( const Collection &collectio
   d->mCurrentCollection = collection;
   d->mTagListHash.clear();
 
-  emit message( KMigratorBase::Info, i18nc( "@info:status", "Starting cache migration for folder %1 of account %2", collection.name(), resource().name() ) );
+  emit message( KMigratorBase::Info, i18nc( "@info:status", "Starting cache migration for folder %1 of account %2", collection.name(), resourceName() ) );
 
   emit status( collection.name() );
 
   if ( d->mImportNewMessages || d->mImportCachedMessages ) {
-    FileStore::ItemFetchJob *job = d->mStore->fetchItems( cache );
+    FileStore::ItemFetchJob *job = store()->fetchItems( cache );
     connect( job, SIGNAL( result( KJob* ) ), SLOT( fetchItemsResult( KJob * ) ) );
     emit status( i18nc( "@info:status foldername", "%1: listing messages...", collection.name() ) );
   } else if ( d->mRemoveDeletedMessages ) {
@@ -590,6 +601,11 @@ void ImapCacheCollectionMigrator::migrationProgress( int processedCollections, i
   }
 }
 
+QString ImapCacheCollectionMigrator::mapRemoteIdFromStore( const QString &storeRemotedId  ) const
+{
+  return '/' + storeRemotedId;
+}
+
 #include "imapcachecollectionmigrator.moc"
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
