@@ -3,6 +3,7 @@
     Copyright (c) 2010 Volker Krause <vkrause@kde.org>
     Copyright (C) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.net
     Author: Kevin Krammer, krake@kdab.com
+    Copyright (C) 2011 Kevin Krammer, kevin.krammer@gmx.at
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -22,9 +23,9 @@
 
 #include "kmailmigrator.h"
 
-#include "messagetag.h"
 
 #ifndef KDEPIM_NO_NEPOMUK
+#include "messagetag.h"
 // avoid metatype.h from interfering
 #include <Nepomuk/Variant>
 #define POP3_METATYPE_H
@@ -38,8 +39,8 @@
 #include "mixedmaildirsettings.h"
 #include "mixedmaildirstore.h"
 #include "emptyresourcecleaner.h"
+#include "imapcacheadapter.h"
 #include "imapcachecollectionmigrator.h"
-#include "imapcachelocalimporter.h"
 #include "localfolderscollectionmigrator.h"
 
 #include "collectionannotationsattribute.h"
@@ -54,17 +55,22 @@ using Akonadi::AgentManager;
 using Akonadi::AgentInstance;
 using Akonadi::AgentInstanceCreateJob;
 
+#include <KIO/CopyJob>
+
 #include <KConfig>
 #include <KConfigGroup>
+#include <KDateTime>
 #include <KDebug>
 #include <KStandardDirs>
 #include <KMessageBox>
 #include <KSharedConfig>
 #include <KCursor>
 #include <kwallet.h>
-#include <QApplication>
 #include <kstringhandler.h>
 #include <KLocalizedString>
+
+#include <QApplication>
+
 using KWallet::Wallet;
 
 using namespace KMail;
@@ -76,7 +82,7 @@ enum MboxLocking { Procmail, MuttDotLock, MuttDotLockPrivileged, MboxNone };
 
 #define SPECIALCOLLECTIONS_LOCK_SERVICE QLatin1String( "org.kde.pim.SpecialCollections" )
 
-static MixedMaildirStore *createCacheStore( const QString &basePath )
+static MixedMaildirStore *createStoreFromBasePath( const QString &basePath )
 {
   MixedMaildirStore *store = 0;
   const QFileInfo baseDir( basePath );
@@ -88,16 +94,21 @@ static MixedMaildirStore *createCacheStore( const QString &basePath )
   return store;
 }
 
+static void backupConfig( const QString &name, const QDir &backupDir )
+{
+  const QString configFile = KStandardDirs::locate( "config", name );
+  if ( !configFile.isEmpty() ) {
+    QFile::copy( configFile, QFileInfo( backupDir, name ).absoluteFilePath() );
+  }
+}
+
 KMailMigrator::KMailMigrator() :
   KMigratorBase(),
   mWallet( 0 ),
-  mDeleteCacheAfterImport( false ),
   mDImapCache( 0 ),
   mImapCache( 0 ),
-  mRunningCacheImporterCount( 0 ),
-  mLocalFoldersDone( false ),
   mForwardResourceNotifications( false ),
-  mLocalCacheImporter( 0 )
+  mImapCacheAdapter( 0 )
 {
   Akonadi::AttributeFactory::registerAttribute<Akonadi::CollectionAnnotationsAttribute>();
   connect( AgentManager::self(), SIGNAL( instanceStatusChanged( Akonadi::AgentInstance ) ),
@@ -117,6 +128,19 @@ void KMailMigrator::migrate()
 {
   emit message( Info, i18n("Beginning KMail migration...") );
 
+  // copy config files to backup locations
+  const KDateTime now = KDateTime::currentLocalDateTime();
+  const QDir backupDir( KGlobal::dirs()->saveLocation( "appdata", now.toString() ) );
+
+  // copy current configs to backup location
+  // strickly speaking neither kmailrc not mailtransports are changed by the migrator
+  // but copy them anyway as the belong conceptually to the KMail1 -> KMail2 migration
+  backupConfig( "kmailrc", backupDir );
+  backupConfig( "mailtransports", backupDir );
+  backupConfig( "emailidentities", backupDir );
+  backupConfig( "kcmkmailsummaryrc", backupDir );
+  backupConfig( "templatesconfigurationrc", backupDir );
+
   KSharedConfigPtr oldConfig = KSharedConfig::openConfig( QLatin1String( "kmailrc" ) );
   mConfig = KSharedConfig::openConfig( QLatin1String( "kmail2rc" ) );
 
@@ -152,16 +176,28 @@ void KMailMigrator::migrate()
   migrateTags();
   migrateRCFiles();
 
-  mAccounts = mConfig->groupList().filter( QRegExp( "Account \\d+" ) );
-
-  if ( evaluateCacheHandlingOptions() ) {
-    mIt = mAccounts.begin();
-    migrateNext();
-  } else { // abort migration
-    deleteLater();
-    qApp->exit( 5 );
+  // copy autosave files if there are any
+  const QString autoSaveDir = KGlobal::dirs()->saveLocation( "data", "kmail/autosave", false );
+  if ( !autoSaveDir.isEmpty() ) {
+    const QString destDir = KGlobal::dirs()->saveLocation( "data", "kmail2" );
+    KIO::CopyJob *job = KIO::copy( KUrl::fromPath( autoSaveDir ),
+                                   KUrl::fromPath( destDir ),
+                                   KIO::HideProgressInfo );
+    job->setAutoSkip( true );
+    job->setWriteIntoExistingDirectories( true );
+    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( autoSaveCopyResult( KJob* ) ) );
   }
 
+  mAccounts = mConfig->groupList().filter( QRegExp( "Account \\d+" ) );
+
+  mIt = mAccounts.begin();
+  migrateNext();
+}
+
+void KMailMigrator::autoSaveCopyResult( KJob *job )
+{
+  kDebug( KDE_DEFAULT_DEBUG_AREA ) << "error=" << job->error()
+    << "text=" << job->errorString();
 }
 
 void KMailMigrator::deleteOldGroup()
@@ -290,8 +326,15 @@ void KMailMigrator::migrateNext()
 
     ++mIt;
   }
-  if ( mIt == mAccounts.end() )
-    migrateLocalFolders();
+  if ( mIt == mAccounts.end() ) {
+    if ( mImapCacheAdapter != 0 ) {
+      emit message( Info, i18n( "Enabling access to the disconnected IMAP cache of the previous KMail version" ) );
+      emit status( QString() );
+      mImapCacheAdapter->start();
+    } else {
+      migrateLocalFolders();
+    }
+  }
 }
 
 void KMailMigrator::migrateLocalFolders()
@@ -299,16 +342,7 @@ void KMailMigrator::migrateLocalFolders()
   if ( migrationState( "LocalFolders" ) == Complete ) {
     emit message( Skip, i18n( "Local folders have already been migrated." ) );
     emit status( QString() );
-    mLocalFoldersDone = true;
-    if ( mRunningCacheImporterCount == 0 ) {
-      migrationDone();
-    } else {
-      kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-      emit status( i18ncp( "@info:status still running background jobs",
-                           "Waiting for one background task to finish",
-                           "Waiting for %1 background tasks to finish",
-                           mRunningCacheImporterCount ) );
-    }
+    migrationDone();
     return;
   }
 
@@ -321,16 +355,7 @@ void KMailMigrator::migrateLocalFolders()
   const QFileInfo fileInfo( mLocalMaildirPath );
   if ( !fileInfo.exists() || !fileInfo.isDir() ) {
     emit status( QString() );
-    mLocalFoldersDone = true;
-    if ( mRunningCacheImporterCount == 0 ) {
-      migrationDone();
-    } else {
-      kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-      emit status( i18ncp( "@info:status still running background jobs",
-                           "Waiting for one background task to finish",
-                           "Waiting for %1 background tasks to finish",
-                           mRunningCacheImporterCount ) );
-    }
+    migrationDone();
   } else {
     kDebug() << mLocalMaildirPath;
 
@@ -522,44 +547,6 @@ void KMailMigrator::connectCollectionMigrator( AbstractCollectionMigrator *migra
            SLOT ( collectionMigratorEmittedNotification() ) );
 }
 
-bool KMailMigrator::evaluateCacheHandlingOptions()
-{
-  bool needsAction = false;
-  Q_FOREACH( const QString &account, mAccounts ) {
-    const KConfigGroup accountGroup( mConfig, account );
-    if ( migrationState( accountGroup.readEntry( "Id" ) ) != Complete ) {
-      const QString type = accountGroup.readEntry( QLatin1String( "Type" ) ).toLower();
-      if ( type == QLatin1String( "dimap" ) ) {
-        needsAction = true;
-        break;
-      }
-    }
-  }
-
-  if ( needsAction ) {
-    QApplication::restoreOverrideCursor();
-
-    const QString message =
-      i18nc( "@info", "<para>Cached IMAP accounts have a local copy of the server's data.</para>"
-                      "<para>These copies can either be kept in local folders, or be deleted "
-                      "after import.</para>"
-                      "<note>Mail that was not imported will always be kept in local folders."
-                      "</note>" );
-
-    int result = KMessageBox::questionYesNoCancel( 0, message, i18n( "KMail 2 Migration" ),
-                                             KGuiItem( i18nc( "@action", "Delete Copies" ) ),
-                                             KGuiItem( i18nc( "@action", "Keep Copies" ) ) );
-    mDeleteCacheAfterImport = ( result == KMessageBox::Yes );
-
-    QApplication::setOverrideCursor( KCursor( QLatin1String( "wait" ), Qt::WaitCursor ) );
-    if ( result == KMessageBox::Cancel ) {
-        return false;
-    }
-  }
-  return true;
-}
-
-
 void KMailMigrator::migratePassword( const QString &idString, const AgentInstance &instance,
                                      const QString &newFolder, const QString& passwordFromFilePassword )
 {
@@ -651,99 +638,20 @@ void KMailMigrator::migrateImapAccount( KJob *job, bool disconnected )
 {
   AgentInstance instance = static_cast< AgentInstanceCreateJob* >( job )->instance();
   mCurrentInstance = instance;
-  KConfigGroup config( mConfig, mCurrentAccount );
-
-  OrgKdeAkonadiImapSettingsInterface *iface = createImapSettingsInterface( instance );
-  if (!iface ) {
-    return;
-  }
 
   AccountConfig conf;
   conf.instance = instance;
   conf.imapAccount = true;
+  conf.disconnectedImap = disconnected;
   mAccountInstance.insert( mCurrentAccount, conf );
 
-  iface->setImapServer( config.readEntry( "host" ) );
-  iface->setImapPort( config.readEntry( "port", 143 ) );
-  iface->setUserName( config.readEntry( "login" ) );
-  if ( config.readEntry( "use-ssl" ).toLower() == "true" )
-    iface->setSafety( "SSL" );
-  else if ( config.readEntry( "use-tls" ).toLower() == "true" )
-    iface->setSafety( "STARTTLS" );
-  else
-    iface->setSafety( "NONE" );
-  const QString authentication = config.readEntry( "auth" ).toUpper();
-  if ( authentication == "LOGIN" )
-    iface->setAuthentication(  MailTransport::Transport::EnumAuthenticationType::LOGIN );
-  else if ( authentication == "PLAIN" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::PLAIN );
-  else if ( authentication == "CRAM-MD5" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::CRAM_MD5 );
-  else if ( authentication == "DIGEST-MD5" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::DIGEST_MD5 );
-  else if ( authentication == "NTLM" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::NTLM );
-  else if ( authentication == "GSSAPI" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::GSSAPI );
-  else if ( authentication == "ANONYMOUS" )
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::ANONYMOUS );
-  else {
-    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::CLEAR );
-  }
-  if ( config.readEntry( "subscribed-folders" ).toLower() == "true" )
-    iface->setSubscriptionEnabled( true );
-
-  // skip interval checking so it doesn't interfere with cache importing
-  iface->setIntervalCheckTime( -1 ); //exclude
-
-  // check-exclude in Account section means that this account should not be included
-  // in manual checks. In KMail UI this is called "Include in manual checks"
-  KConfigGroup resourceGroup( mConfig, QString::fromLatin1( "Resource %1" ).arg( instance.identifier() ) );
-  resourceGroup.writeEntry( "IncludeInManualChecks", !config.readEntry( "check-exclude", false ) );
-  const KConfigGroup generalGroup( mConfig, "General" );
-  resourceGroup.writeEntry( "CheckOnStartup", generalGroup.readEntry( "checkmail-startup", false ) );
-  resourceGroup.writeEntry( "OfflineOnShutdown", true );
-
-  iface->setSieveSupport( config.readEntry( "sieve-support", false ) );
-  iface->setSieveReuseConfig( config.readEntry( "sieve-reuse-config", true ) );
-  iface->setSievePort( config.readEntry( "sieve-port", 2000 ) );
-  iface->setSieveAlternateUrl( config.readEntry( "sieve-alternate-url" ) );
-  iface->setSieveVacationFilename( config.readEntry( "sieve-vacation-filename", "kmail-vacation.siv" ) );
-  iface->setDisconnectedModeEnabled( disconnected );
-  if ( !disconnected ) {
-    iface->setAutomaticExpungeEnabled( config.readEntry("auto-expunge", true ) );
-  }
-  const bool useDefaultIdentity = config.readEntry( "use-default-identity", true );
-  iface->setUseDefaultIdentity( useDefaultIdentity );
-  if ( !useDefaultIdentity )
-    iface->setAccountIdentity( config.readEntry( "identity-id" ).toInt() );
-
-  // make sure the config is saved
-  iface->writeConfig();
-
-  QString encpasswd ;
-  if ( config.readEntry( "store-passwd", false ) ) {
-    encpasswd = config.readEntry( "pass" );
-    if ( encpasswd.isEmpty() ) {
-      encpasswd = config.readEntry( "passwd" );
-      if ( !encpasswd.isEmpty() )
-        encpasswd = importPassword( encpasswd );
-    }
-
-    if ( !encpasswd.isEmpty() ) {
-      encpasswd = KStringHandler::obscure( encpasswd );
-    }
-    config.deleteEntry( "store-passwd" );
-    config.deleteEntry( "passwd" );
-  }
-  migratePassword( config.readEntry( "Id" ), instance, "imap", encpasswd );
+  KConfigGroup config( mConfig, mCurrentAccount );
 
   const QString nameAccount = config.readEntry( "Name" );
   instance.setName( nameAccount );
   emit status( nameAccount );
-  instance.reconfigure();
 
-  ImapCacheCollectionMigrator::MigrationOptions options = ImapCacheCollectionMigrator::ImportCachedMessages;
+  ImapCacheCollectionMigrator::MigrationOptions options = ImapCacheCollectionMigrator::ImportNewMessages;
   if ( disconnected ) {
     const KConfigGroup dimapConfig( KGlobal::config(), QLatin1String( "Disconnected IMAP" ) );
     if ( dimapConfig.isValid() ) {
@@ -765,26 +673,32 @@ void KMailMigrator::migrateImapAccount( KJob *job, bool disconnected )
 
   MixedMaildirStore *store = 0;
   if ( disconnected ) {
+#if 0
     if ( mDeleteCacheAfterImport ) {
       options |= ImapCacheCollectionMigrator::DeleteImportedMessages;
     }
+#endif
     if ( mDImapCache == 0 ){
-      mDImapCache = createCacheStore( KStandardDirs::locateLocal( "data", QLatin1String( "kmail/dimap" ) ) );
+      mDImapCache = createStoreFromBasePath( KStandardDirs::locateLocal( "data", QLatin1String( "kmail/dimap" ) ) );
     }
     store = mDImapCache;
   } else {
     if ( mImapCache == 0 ){
-      mImapCache = createCacheStore( KStandardDirs::locateLocal( "data", QLatin1String( "kmail/imap" ) ) );
+      mImapCache = createStoreFromBasePath( KStandardDirs::locateLocal( "data", QLatin1String( "kmail/imap" ) ) );
     }
     store = mImapCache;
   }
 
-  ImapCacheCollectionMigrator *collectionMigrator = new ImapCacheCollectionMigrator( instance, store, this );
+  ImapCacheCollectionMigrator *collectionMigrator = new ImapCacheCollectionMigrator( instance, nameAccount, store, this );
   QString topLevelFolder = config.readEntry( "Folder" );
   if ( topLevelFolder.isEmpty() ) {
     topLevelFolder = config.readEntry( "Id" );
   }
-  collectionMigrator->setTopLevelFolder( topLevelFolder );
+
+  const QString topLevelRemoteId =
+    "imap://" + config.readEntry( "login" ) + '@' + config.readEntry( "host" ) + '/';
+
+  collectionMigrator->setTopLevelFolder( topLevelFolder, nameAccount, topLevelRemoteId );
   collectionMigrator->setMigrationOptions( options );
 
   kDebug() << "Starting IMAP collection migration: options="
@@ -797,26 +711,22 @@ void KMailMigrator::migrateImapAccount( KJob *job, bool disconnected )
   if ( config.readEntry( "locally-subscribed-folders", false ) ) {
     collectionMigrator->setUnsubscribedImapFolders( config.readEntry( "locallyUnsubscribedFolders", QStringList() ) );
   }
-  config.deleteEntry( "locally-subscribed-folders" );
-  config.deleteEntry( "locallyUnsubscribedFolders" );
-
-  config.deleteEntry( "capabilities" );
 
   if ( disconnected && store ) {
-    mLocalCacheImporter = new ImapCacheLocalImporter( store, this );
-    mLocalCacheImporter->setTopLevelFolder( collectionMigrator->topLevelFolder() );
-    mLocalCacheImporter->setAccountName( nameAccount );
-
-    connect( mLocalCacheImporter, SIGNAL( importFinished( Akonadi::AgentInstance, QString ) ),
-             SLOT( imapCacheImportFinished( Akonadi::AgentInstance, QString ) ) );
-  } else {
-    mLocalCacheImporter = 0;
+    if ( mImapCacheAdapter == 0 ) {
+      mImapCacheAdapter = new ImapCacheAdapter( store, this );
+      connect( mImapCacheAdapter, SIGNAL( finished( int, QString ) ),
+               SLOT( imapCacheAdaptionFinished( int, QString ) ) );
+    }
+    mImapCacheAdapter->addAccount( topLevelFolder, nameAccount );
   }
 
   connect( collectionMigrator, SIGNAL( migrationFinished( Akonadi::AgentInstance, QString ) ),
            SLOT( imapFoldersMigrationFinished( Akonadi::AgentInstance, QString ) ) );
 
   connectCollectionMigrator( collectionMigrator );
+
+  collectionMigrator->startMigration();
 }
 
 void KMailMigrator::pop3AccountCreated( KJob *job )
@@ -1069,21 +979,6 @@ void KMailMigrator::localMaildirCreated( KJob *job )
   AgentInstance instance = static_cast< AgentInstanceCreateJob* >( job )->instance();
   mCurrentInstance = instance;
 
-  OrgKdeAkonadiMixedMaildirSettingsInterface *iface = new OrgKdeAkonadiMixedMaildirSettingsInterface(
-    "org.freedesktop.Akonadi.Resource." + instance.identifier(),
-    "/Settings", QDBusConnection::sessionBus(), this );
-
-  if (!iface->isValid() ) {
-    migrationFailed( i18n("Failed to obtain D-Bus interface for remote configuration."), instance );
-    delete iface;
-    return;
-  }
-
-  iface->setPath( mLocalMaildirPath );
-
-  // make sure the config is saved
-  iface->writeConfig();
-
   // do not include KMail folders in manual checks by default
   KConfigGroup resourceGroup( mConfig, QString::fromLatin1( "Resource %1" ).arg( instance.identifier() ) );
   resourceGroup.writeEntry( "IncludeInManualChecks", false );
@@ -1138,9 +1033,11 @@ void KMailMigrator::localMaildirCreated( KJob *job )
 
   instance.setName( instanceName );
   emit status( instanceName );
-  instance.reconfigure();
 
-  LocalFoldersCollectionMigrator *collectionMigrator = new LocalFoldersCollectionMigrator( instance, this );
+  MixedMaildirStore *store = createStoreFromBasePath( mLocalMaildirPath );
+
+  LocalFoldersCollectionMigrator *collectionMigrator = new LocalFoldersCollectionMigrator( instance, instanceName, store, this );
+  collectionMigrator->setTopLevelFolder( QString(), instanceName );
   collectionMigrator->setKMailConfig( mConfig );
   collectionMigrator->setEmailIdentityConfig( mEmailIdentityConfig );
   collectionMigrator->setKcmKmailSummaryConfig( mKcmKmailSummaryConfig );
@@ -1149,10 +1046,30 @@ void KMailMigrator::localMaildirCreated( KJob *job )
   connect( collectionMigrator, SIGNAL( migrationFinished( Akonadi::AgentInstance, QString ) ),
            SLOT( localFoldersMigrationFinished( Akonadi::AgentInstance, QString ) ) );
   connectCollectionMigrator( collectionMigrator );
+
+  collectionMigrator->startMigration();
 }
 
 void KMailMigrator::localFoldersMigrationFinished( const AgentInstance &instance, const QString &error )
 {
+  OrgKdeAkonadiMixedMaildirSettingsInterface *iface = new OrgKdeAkonadiMixedMaildirSettingsInterface(
+    "org.freedesktop.Akonadi.Resource." + instance.identifier(),
+    "/Settings", QDBusConnection::sessionBus(), this );
+
+  if (!iface->isValid() ) {
+    migrationFailed( i18n("Failed to obtain D-Bus interface for remote configuration."), instance );
+    delete iface;
+    return;
+  }
+
+  iface->setPath( mLocalMaildirPath );
+
+  // make sure the config is saved
+  iface->writeConfig();
+
+  AgentInstance resource = instance;
+  resource.reconfigure();
+
   if ( error.isEmpty() ) {
     KConfigGroup config( KGlobal::config(), QLatin1String( "SpecialMailCollections" ) );
     if ( config.readEntry( QLatin1String( "TakeOverIfDefaultIsTotallyEmpty" ), false ) ) {
@@ -1174,23 +1091,14 @@ void KMailMigrator::localFoldersMigrationFinished( const AgentInstance &instance
       }
     }
 
-    mLocalFoldersDone = true;
     mCurrentInstance = AgentInstance();
 
     setMigrationState( "LocalFolders", Complete, instance.identifier(), "LocalFolders" );
     emit message( Success, i18n( "Local folders migrated successfully." ) );
     emit status( QString() );
-    if ( mRunningCacheImporterCount == 0 ) {
-      migrationDone();
-    } else {
-      kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-      emit status( i18ncp( "@info:status still running background jobs",
-                           "Waiting for one background task to finish",
-                           "Waiting for %1 background tasks to finish",
-                           mRunningCacheImporterCount ) );
-    }
+    migrationDone();
+
   } else {
-    mLocalFoldersDone = true;
     mCurrentInstance = AgentInstance();
 
     migrationFailed( error, instance );
@@ -1201,73 +1109,111 @@ void KMailMigrator::imapFoldersMigrationFinished( const AgentInstance &instance,
 {
   kDebug() << "imapMigrationFinished: instance=" << instance.identifier()
            << "error=" << error;
-  if ( error.isEmpty() ) {
-    OrgKdeAkonadiImapSettingsInterface *iface = createImapSettingsInterface( instance );
-    if (!iface) {
-      return;
-    }
 
-    // enable interval checking in case this had been configured
-    const KConfigGroup config( mConfig, mCurrentAccount );
-    const int checkInterval = config.readEntry( "check-interval", 0 );
-    if ( checkInterval != 0 ) {
-      iface->setIntervalCheckEnabled( true );
-      iface->setIntervalCheckTime( checkInterval );
-      // make sure the config is saved
-      iface->writeConfig();
-      instance.reconfigure();
-    }
-
-    // if we don't keep the whole cache, run this as a background job
-    // the migration has not failed, so there are at worst only a few messages left to
-    // to import
-    if ( mDeleteCacheAfterImport && mLocalCacheImporter != 0 ) {
-      ++mRunningCacheImporterCount;
-
-      emit message( Info,
-                    i18ncp( "@info:status",
-                            "Scheduling background task for recovering non-imported messages.",
-                            "Scheduling background task for recovering non-imported messages. "
-                            "Now at a total of %1 such tasks",
-                             mRunningCacheImporterCount ) );
-
-      migrationCompleted( instance );
-      mLocalCacheImporter->startImport();
-      mLocalCacheImporter = 0;
-    } else if ( mLocalCacheImporter != 0 ) {
-      migrationCompleted( instance, false );
-
-      // we are running the cache importer as a normal migration task, connect progress signals
-      connect( mLocalCacheImporter, SIGNAL( status( QString ) ), SIGNAL ( status( QString ) ) );
-      connect( mLocalCacheImporter, SIGNAL( progress( int ) ), SIGNAL ( progress( int ) ) );
-      connect( mLocalCacheImporter, SIGNAL( progress( int, int, int ) ),
-               SIGNAL ( progress( int, int, int ) ) );
-
-      emit message( Info, i18nc( "@info:status importing cached messages of a DIMAP account",
-                                 "Starting local cache import for %1",
-                                 mLocalCacheImporter->accountName() ) );
-    } else {
-      migrationCompleted( instance );
-    }
-  } else {
-    migrationFailed( error, mLocalCacheImporter == 0, instance );
-
-    if ( mLocalCacheImporter != 0 ) {
-      mLocalCacheImporter->setProperty( "imapImportFailed", QVariant::fromValue<bool>( true ) );
-
-      // we are running the cache importer as a normal migration task, connect progress signals
-      connect( mLocalCacheImporter, SIGNAL( status( QString ) ), SIGNAL ( status( QString ) ) );
-      connect( mLocalCacheImporter, SIGNAL( progress( int ) ), SIGNAL ( progress( int ) ) );
-      connect( mLocalCacheImporter, SIGNAL( progress( int, int, int ) ),
-               SIGNAL ( progress( int, int, int ) ) );
-      emit message( Info, i18nc( "@info:status importing cached messages of a DIMAP account",
-                                  "Starting local cache import for %1",
-                                  mLocalCacheImporter->accountName() ) );
-    }
+  OrgKdeAkonadiImapSettingsInterface *iface = createImapSettingsInterface( instance );
+  if (!iface ) {
+    return;
   }
 
-  if ( mLocalCacheImporter != 0 ) {
-    mLocalCacheImporter->startImport();
+  const bool disconnected = mAccountInstance[ mCurrentAccount ].disconnectedImap;
+
+  KConfigGroup config( mConfig, mCurrentAccount );
+  const QString nameAccount = config.readEntry( "Name" );
+
+  iface->setImapServer( config.readEntry( "host" ) );
+  iface->setImapPort( config.readEntry( "port", 143 ) );
+  iface->setUserName( config.readEntry( "login" ) );
+  if ( config.readEntry( "use-ssl" ).toLower() == "true" )
+    iface->setSafety( "SSL" );
+  else if ( config.readEntry( "use-tls" ).toLower() == "true" )
+    iface->setSafety( "STARTTLS" );
+  else
+    iface->setSafety( "NONE" );
+  const QString authentication = config.readEntry( "auth" ).toUpper();
+  if ( authentication == "LOGIN" )
+    iface->setAuthentication(  MailTransport::Transport::EnumAuthenticationType::LOGIN );
+  else if ( authentication == "PLAIN" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::PLAIN );
+  else if ( authentication == "CRAM-MD5" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::CRAM_MD5 );
+  else if ( authentication == "DIGEST-MD5" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::DIGEST_MD5 );
+  else if ( authentication == "NTLM" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::NTLM );
+  else if ( authentication == "GSSAPI" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::GSSAPI );
+  else if ( authentication == "ANONYMOUS" )
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::ANONYMOUS );
+  else {
+    iface->setAuthentication( MailTransport::Transport::EnumAuthenticationType::CLEAR );
+  }
+  if ( config.readEntry( "subscribed-folders" ).toLower() == "true" )
+    iface->setSubscriptionEnabled( true );
+
+  // skip interval checking so it doesn't interfere with cache importing
+  iface->setIntervalCheckTime( -1 ); //exclude
+
+  // check-exclude in Account section means that this account should not be included
+  // in manual checks. In KMail UI this is called "Include in manual checks"
+  KConfigGroup resourceGroup( mConfig, QString::fromLatin1( "Resource %1" ).arg( instance.identifier() ) );
+  resourceGroup.writeEntry( "IncludeInManualChecks", !config.readEntry( "check-exclude", false ) );
+  const KConfigGroup generalGroup( mConfig, "General" );
+  resourceGroup.writeEntry( "CheckOnStartup", generalGroup.readEntry( "checkmail-startup", false ) );
+  resourceGroup.writeEntry( "OfflineOnShutdown", true );
+
+  iface->setSieveSupport( config.readEntry( "sieve-support", false ) );
+  iface->setSieveReuseConfig( config.readEntry( "sieve-reuse-config", true ) );
+  iface->setSievePort( config.readEntry( "sieve-port", 2000 ) );
+  iface->setSieveAlternateUrl( config.readEntry( "sieve-alternate-url" ) );
+  iface->setSieveVacationFilename( config.readEntry( "sieve-vacation-filename", "kmail-vacation.siv" ) );
+  iface->setDisconnectedModeEnabled( disconnected );
+  if ( !disconnected ) {
+    iface->setAutomaticExpungeEnabled( config.readEntry("auto-expunge", true ) );
+  }
+  const bool useDefaultIdentity = config.readEntry( "use-default-identity", true );
+  iface->setUseDefaultIdentity( useDefaultIdentity );
+  if ( !useDefaultIdentity )
+    iface->setAccountIdentity( config.readEntry( "identity-id" ).toInt() );
+
+  QString encpasswd ;
+  if ( config.readEntry( "store-passwd", false ) ) {
+    encpasswd = config.readEntry( "pass" );
+    if ( encpasswd.isEmpty() ) {
+      encpasswd = config.readEntry( "passwd" );
+      if ( !encpasswd.isEmpty() )
+        encpasswd = importPassword( encpasswd );
+    }
+
+    if ( !encpasswd.isEmpty() ) {
+      encpasswd = KStringHandler::obscure( encpasswd );
+    }
+    config.deleteEntry( "store-passwd" );
+    config.deleteEntry( "passwd" );
+  }
+  migratePassword( config.readEntry( "Id" ), instance, "imap", encpasswd );
+
+  // enable interval checking in case this had been configured
+  const int checkInterval = config.readEntry( "check-interval", 0 );
+  if ( checkInterval != 0 ) {
+    iface->setIntervalCheckEnabled( true );
+    iface->setIntervalCheckTime( checkInterval );
+  }
+
+  // make sure the config is saved
+  iface->writeConfig();
+
+  AgentInstance resource = instance;
+  resource.reconfigure();
+
+  config.deleteEntry( "locally-subscribed-folders" );
+  config.deleteEntry( "locallyUnsubscribedFolders" );
+
+  config.deleteEntry( "capabilities" );
+
+  if ( error.isEmpty() ) {
+    migrationCompleted( instance );
+  } else {
+    migrationFailed( error, true, instance );
   }
 }
 
@@ -1302,15 +1248,7 @@ void KMailMigrator::collectionMigratorMessage( int type, const QString &msg )
 
 void KMailMigrator::collectionMigratorFinished()
 {
-  if ( mLocalFoldersDone && mRunningCacheImporterCount > 0 ) {
-    kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-    emit status( i18ncp( "@info:status still running background jobs",
-                         "Waiting for one background task to finish",
-                         "Waiting for %1 background tasks to finish",
-                         mRunningCacheImporterCount ) );
-  } else {
-    emit status( QString() );
-  }
+  emit status( QString() );
 }
 
 void KMailMigrator::collectionMigratorEmittedNotification()
@@ -1348,94 +1286,13 @@ void KMailMigrator::instanceProgressChanged( const AgentInstance &instance )
   emit progress( 0, 100, instance.progress() );
 }
 
-void KMailMigrator::imapCacheImportFinished( const Akonadi::AgentInstance &instance, const QString &error )
+void KMailMigrator::imapCacheAdaptionFinished( int messageType, const QString &messageText )
 {
-  kDebug() << "CacheImport into" << instance.identifier()
-           << "finished (error=" << error << "), "
-           << ( mRunningCacheImporterCount - 1 ) << "still running";
+  emit message( (KMigratorBase::MessageType)messageType, messageText );
 
-  const bool isBackgroundTask = ( sender() != mLocalCacheImporter );
+  mImapCacheAdapter = 0;
 
-  if ( !error.isEmpty() ) {
-    if ( isBackgroundTask ) {
-        --mRunningCacheImporterCount;
-        kDebug() << "running cache importers down to" << mRunningCacheImporterCount;
-    }
-
-    emit message( Error, error );
-    if ( mLocalFoldersDone ) {
-      kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-      emit status( i18ncp( "@info:status still running background jobs",
-                           "Waiting for one background task to finish",
-                           "Waiting for %1 background tasks to finish",
-                           mRunningCacheImporterCount ) );
-    }
-  } else {
-    // delete empty folders unless we explicitly keep the local cache or
-    // the actual IMAP import has failed
-    bool doCleanup = mDeleteCacheAfterImport;
-    if ( !isBackgroundTask ) {
-      const QVariant importFailedVar = mLocalCacheImporter->property( "imapImportFailed" );
-      if ( importFailedVar.isValid() || importFailedVar.value<bool>() ) {
-        doCleanup = false;
-      }
-    }
-    if ( doCleanup ) {
-      EmptyResourceCleaner *cleaner = new EmptyResourceCleaner( instance, this );
-      cleaner->setCleaningOptions( EmptyResourceCleaner::DeleteEmptyCollections |
-                                   EmptyResourceCleaner::DeleteEmptyResource );
-      connect( cleaner, SIGNAL( cleanupFinished( Akonadi::AgentInstance ) ),
-               SLOT( imapCacheCleanupFinished( Akonadi::AgentInstance ) ) );
-
-      // if the cache importer ran as a foreground task, the cleaner always runs as
-      // a background task
-      if ( !isBackgroundTask ) {
-        ++mRunningCacheImporterCount;
-      }
-    } else {
-      if ( isBackgroundTask ) {
-        --mRunningCacheImporterCount;
-        kDebug() << "running cache importers down to" << mRunningCacheImporterCount;
-      } else {
-        emit message( Success, i18nc( "@info:status", "Local cache import for %1 completed",
-                                      mLocalCacheImporter->accountName() ) );
-      }
-    }
-  }
-
-  if ( mRunningCacheImporterCount == 0 && mLocalFoldersDone ) {
-    migrationDone();
-  } else {
-    if ( mLocalFoldersDone ) {
-      kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-      emit status( i18ncp( "@info:status still running background jobs",
-                           "Waiting for one background task to finish",
-                           "Waiting for %1 background tasks to finish",
-                           mRunningCacheImporterCount ) );
-    }
-
-    if ( !isBackgroundTask ) {
-      migrateNext();
-    }
-  }
-}
-
-void KMailMigrator::imapCacheCleanupFinished( const Akonadi::AgentInstance &instance )
-{
-  --mRunningCacheImporterCount;
-
-  kDebug() << "CacheCleanup of" << instance.identifier() << "finished, "
-           << mRunningCacheImporterCount << "still running";
-
-  if ( mRunningCacheImporterCount == 0 && mLocalFoldersDone ) {
-    migrationDone();
-  } else if ( mLocalFoldersDone ) {
-    kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-    emit status( i18ncp( "@info:status still running background jobs",
-                         "Waiting for one background task to finish",
-                         "Waiting for %1 background tasks to finish",
-                         mRunningCacheImporterCount ) );
-  }
+  migrateNext();
 }
 
 void KMailMigrator::specialColDefaultResourceCheckFinished( const AgentInstance &instance )
@@ -1465,21 +1322,12 @@ void KMailMigrator::specialColDefaultResourceCheckFinished( const AgentInstance
     QDBusConnection::sessionBus().unregisterService( SPECIALCOLLECTIONS_LOCK_SERVICE );
   }
 
-  mLocalFoldersDone = true;
   mCurrentInstance = AgentInstance();
 
   setMigrationState( "LocalFolders", Complete, localFoldersIdentifier, "LocalFolders" );
   emit message( Success, i18n( "Local folders migrated successfully." ) );
   emit status( QString() );
-  if ( mRunningCacheImporterCount == 0 ) {
-    migrationDone();
-  } else {
-    kDebug() << "Waiting for" << mRunningCacheImporterCount << "background tasks";
-    emit status( i18ncp( "@info:status still running background jobs",
-                         "Waiting for one background task to finish",
-                         "Waiting for %1 background tasks to finish",
-                         mRunningCacheImporterCount ) );
-  }
+  migrationDone();
 }
 
 QString KMailMigrator::importPassword(const QString &aStr)
