@@ -30,6 +30,7 @@
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kdeversion.h>
 
 #include <QtCore/qplugin.h>
 
@@ -69,7 +70,11 @@ void SerializerPluginKCalCore::serialize(const Item & item, const QByteArray & l
   Incidence::Ptr i = item.payload<Incidence::Ptr>();
   // ### I guess this can be done without hardcoding stuff
   data.write( "BEGIN:VCALENDAR\nPRODID:-//K Desktop Environment//NONSGML libkcal 3.2//EN\nVERSION:2.0\n" );
+#if KDE_IS_VERSION( 4, 6, 40 )
+  data.write( mFormat.toRawString( i ) );
+#else
   data.write( mFormat.toString( i ).toUtf8() );
+#endif
   data.write( "\nEND:VCALENDAR" );
 }
 
