@@ -63,11 +63,16 @@ DavGroupwareResource::DavGroupwareResource( const QString &id )
   AttributeFactory::registerAttribute<EntityDisplayAttribute>();
   AttributeFactory::registerAttribute<DavProtocolAttribute>();
 
+  setNeedsNetwork( true );
+
   mDavCollectionRoot.setParentCollection( Collection::root() );
   mDavCollectionRoot.setName( identifier() );
   mDavCollectionRoot.setRemoteId( identifier() );
   mDavCollectionRoot.setContentMimeTypes( QStringList() << Collection::mimeType() );
 
+  EntityDisplayAttribute *attribute = mDavCollectionRoot.attribute<EntityDisplayAttribute>( Collection::AddIfMissing );
+  attribute->setIconName( QLatin1String( "folder-remote" ) );
+
   int refreshInterval = Settings::self()->refreshInterval();
   if ( refreshInterval == 0 )
     refreshInterval = -1;
@@ -85,7 +90,6 @@ DavGroupwareResource::DavGroupwareResource( const QString &id )
   changeRecorder()->itemFetchScope().setAncestorRetrieval( ItemFetchScope::All );
 
   Settings::self()->setWinId( winIdForDialogs() );
-  synchronize();
 }
 
 DavGroupwareResource::~DavGroupwareResource()
@@ -168,8 +172,8 @@ void DavGroupwareResource::retrieveCollections()
 
   DavCollectionsMultiFetchJob *job = new DavCollectionsMultiFetchJob( Settings::self()->configuredDavUrls() );
   connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveCollectionsFinished( KJob* ) ) );
-  connect( job, SIGNAL( collectionDiscovered( const QString&, const QString& ) ),
-           SLOT( onCollectionDiscovered( const QString&, const QString& ) ) );
+  connect( job, SIGNAL( collectionDiscovered( int, const QString&, const QString& ) ),
+           SLOT( onCollectionDiscovered( int, const QString&, const QString& ) ) );
   job->start();
 }
 
@@ -185,6 +189,14 @@ void DavGroupwareResource::retrieveItems( const Akonadi::Collection &collection
 
   const DavUtils::DavUrl davUrl = Settings::self()->davUrlFromCollectionUrl( collection.remoteId() );
 
+  if ( !davUrl.url().isValid() ) {
+    kError() << "Can't find a configured URL, collection.remoteId() is " << collection.remoteId();
+    // TODO: make this string translatable
+    cancelTask( "Items requested for an unknown collection" );
+    //Q_ASSERT_X( false, "DavGroupwareResource::retrieveItems", "Url is invalid" );
+    return;
+  }
+
   DavItemsListJob *job = new DavItemsListJob( davUrl );
   job->setProperty( "collection", QVariant::fromValue( collection ) );
   connect( job, SIGNAL( result( KJob* ) ), SLOT( onRetrieveItemsFinished( KJob* ) ) );
@@ -359,6 +371,22 @@ void DavGroupwareResource::itemRemoved( const Akonadi::Item &item )
   job->start();
 }
 
+void DavGroupwareResource::aboutToQuit()
+{
+  Settings::self()->writeConfig();
+}
+
+void DavGroupwareResource::doSetOnline( bool online )
+{
+  kDebug() << "Resource changed online status to" << online;
+
+  if ( online ) {
+    synchronize();
+  }
+
+  ResourceBase::doSetOnline( online );
+}
+
 void DavGroupwareResource::onCollectionRemovedFinished( KJob *job )
 {
   if ( job->error() ) {
@@ -401,6 +429,9 @@ void DavGroupwareResource::onRetrieveCollectionsFinished( KJob *job )
     QStringList mimeTypes;
 
     const DavCollection::ContentTypes contentTypes = davCollection.contentTypes();
+    if ( contentTypes & DavCollection::Calendar )
+      mimeTypes << QLatin1String( "text/calendar" );
+
     if ( contentTypes & DavCollection::Events )
       mimeTypes << KCalCore::Event::eventMimeType();
 
@@ -410,15 +441,14 @@ void DavGroupwareResource::onRetrieveCollectionsFinished( KJob *job )
     if ( contentTypes & DavCollection::Contacts )
       mimeTypes << KABC::Addressee::mimeType();
 
-    // CalDAV collections can contain events and todos, however in onRetrieveItemsFinished() we
-    // can't figure out what type of items are provided. For this case we'll set the
-    // temporary mime type to 'text/calendar' and fix it in onRetrieveItemFinished() where we can
-    // find out the mime type from the payload. The Akonadi collections from CalDAV need content
-    // mime type 'text/calendar' to temporary store the items.
-    if ( (contentTypes & DavCollection::Events) && (contentTypes & DavCollection::Todos) ) // only valid for CalDAV
-      mimeTypes << QLatin1String( "text/calendar" );
+    if ( contentTypes & DavCollection::FreeBusy )
+      mimeTypes << KCalCore::FreeBusy::freeBusyMimeType();
+
+    if ( contentTypes & DavCollection::Journal )
+      mimeTypes << KCalCore::Journal::journalMimeType();
 
     collection.setContentMimeTypes( mimeTypes );
+    setCollectionIcon( collection /*by-ref*/ );
 
     DavProtocolAttribute *protoAttr = collection.attribute<DavProtocolAttribute>( Collection::AddIfMissing );
     protoAttr->setDavProtocol( davCollection.protocol() );
@@ -453,12 +483,16 @@ void DavGroupwareResource::onRetrieveItemsFinished( KJob *job )
     const QStringList contentMimeTypes = collection.contentMimeTypes();
     if ( contentMimeTypes.contains( KABC::Addressee::mimeType() ) )
       item.setMimeType( KABC::Addressee::mimeType() );
-    else if ( contentMimeTypes.contains( "text/calendar" ) )
-      item.setMimeType( "text/calendar" );
+    else if ( contentMimeTypes.contains( QLatin1String( "text/calendar" ) ) )
+      item.setMimeType( QLatin1String( "text/calendar" ) );
     else if ( contentMimeTypes.contains( KCalCore::Event::eventMimeType() ) )
       item.setMimeType( KCalCore::Event::eventMimeType() );
     else if ( contentMimeTypes.contains( KCalCore::Todo::todoMimeType() ) )
       item.setMimeType( KCalCore::Todo::todoMimeType() );
+    else if ( contentMimeTypes.contains( KCalCore::FreeBusy::freeBusyMimeType() ) )
+      item.setMimeType( KCalCore::FreeBusy::freeBusyMimeType() );
+    else if ( contentMimeTypes.contains( KCalCore::Journal::journalMimeType() ) )
+      item.setMimeType( KCalCore::Journal::journalMimeType() );
 
     if ( mEtagCache.etagChanged( item.remoteId(), davItem.etag() ) ) {
       // Only clear the payload (and therefor trigger a refetch from the backend) if we
@@ -671,9 +705,9 @@ void DavGroupwareResource::onItemRemovedFinished( KJob *job )
   changeProcessed();
 }
 
-void DavGroupwareResource::onCollectionDiscovered( const QString &collection, const QString &config )
+void DavGroupwareResource::onCollectionDiscovered( int protocol, const QString &collection, const QString &config )
 {
-  Settings::self()->addCollectionUrlMapping( collection, config );
+  Settings::self()->addCollectionUrlMapping( DavUtils::Protocol( protocol ), collection, config );
 }
 
 bool DavGroupwareResource::configurationIsValid()
@@ -701,6 +735,24 @@ bool DavGroupwareResource::configurationIsValid()
   return true;
 }
 
+/*static*/
+void DavGroupwareResource::setCollectionIcon( Akonadi::Collection &collection )
+{
+  const QStringList mimeTypes = collection.contentMimeTypes();
+  if ( mimeTypes.count() == 1 ) {
+    QHash<QString,QString> mapping;
+    mapping.insert( KCalCore::Event::eventMimeType(), QLatin1String( "view-calendar" ) );
+    mapping.insert( KCalCore::Todo::todoMimeType(), QLatin1String( "view-calendar-tasks" ) );
+    mapping.insert( KCalCore::Journal::journalMimeType(), QLatin1String( "view-pim-journal" ) );
+    mapping.insert( KABC::Addressee::mimeType(), QLatin1String( "view-pim-contacts" ) );
+
+    if ( mapping.contains( mimeTypes.first() ) ) {
+      EntityDisplayAttribute *attribute = collection.attribute<EntityDisplayAttribute>( Collection::AddIfMissing );
+      attribute->setIconName( mapping.value( mimeTypes.first() ) );
+    }
+  }
+}
+
 AKONADI_RESOURCE_MAIN( DavGroupwareResource )
 
 #include "davgroupwareresource.moc"
